@@ -568,8 +568,20 @@ get_oid_name (const char *oid)
     return "dhpublicnumber";
   else if (!strcmp ("2.16.840.1.101.2.1.1.22", oid))
     return "id-keyExchangeAlgorithm";
+  else if (!strcmp ("1.2.840.10045.1.1", oid))
+    return "prime-field";
   else if (!strcmp ("1.2.840.10045.2.1", oid))
     return "id-ecPublicKey";
+  else if (!strcmp ("1.2.840.10045.4.1", oid))
+    return "ecdsa-with-SHA1";
+  else if (!strcmp ("1.2.840.10045.4.3.1", oid))
+    return "ecdsa-with-SHA224";
+  else if (!strcmp ("1.2.840.10045.4.3.2", oid))
+    return "ecdsa-with-SHA256";
+  else if (!strcmp ("1.2.840.10045.4.3.3", oid))
+    return "ecdsa-with-SHA384";
+  else if (!strcmp ("1.2.840.10045.4.3.4", oid))
+    return "ecdsa-with-SHA512";
   else if (!strcmp ("1.3.132.1.12", oid))
     return "id-ecDH";
   else if (!strcmp ("1.2.840.10045.2.13", oid))
@@ -634,6 +646,18 @@ get_oid_name (const char *oid)
     return "sect571k1";
   else if (!strcmp ("1.3.132.0.39", oid))
     return "sect571r1";
+  else if (!strcmp ("2.16.840.1.101.3.4.3.1", oid))
+    return "id-dsa-with-sha224";
+  else if (!strcmp ("2.16.840.1.101.3.4.3.2", oid))
+    return "id-dsa-with-sha256";
+  else if (!strcmp ("2.16.840.1.101.3.4.2.1", oid))
+    return "sha256";
+  else if (!strcmp ("2.16.840.1.101.3.4.2.2", oid))
+    return "sha384";
+  else if (!strcmp ("2.16.840.1.101.3.4.2.3", oid))
+    return "sha512";
+  else if (!strcmp ("2.16.840.1.101.3.4.2.4", oid))
+    return "sha224";
   else
     return NULL;
 }
@@ -737,15 +761,21 @@ get_name (const char *string)
  *
  * - @a image       Return the entire certificate as binary data.
  *
- * - @a algorithm-name: Return the algorithm name. Get the OID of
- *                      the digest algorithm and translated to a name
- *                      from a list from Wireshark.
- *                      See epan/dissectors/packet-pkcs1.c
+ * - @a algorithm-name  Same as signature-algorithm-name. TODO: Remove it and
+ *                      leave only signature-algorithm-name.
  *
- * - @a modulus:     Return the RSA public key's modulus found in the
+ * - @a signature-algorithm-name  Return the algorithm name used to sign the
+ *                                certificate. Get the OID of the digest
+ *                                algorithm and translated to a name from a
+ *                                list from Wireshark.
+ *                                See epan/dissectors/packet-pkcs1.c
+ *
+ * - @a public-key-algorithm-name  Return the algorithm name of the public key.
+ *
+ * - @a modulus      Return the RSA public key's modulus found in the
  *                   structure of the given cert.
  *
- * - @a exponent:   Return the RSA public key's exponent found in
+ * - @a exponent    Return the RSA public key's exponent found in
  *                  the structure of the given cert.
  *
  * - @a key-size    Return the size to hold the parameters size in bits.
@@ -893,7 +923,8 @@ nasl_cert_query (lex_ctxt *lexic)
           memcpy (retc->x.str_val, der, derlen);
         }
     }
-  else if (!strcmp (command, "algorithm-name"))
+  else if (!strcmp (command, "algorithm-name")
+           || !strcmp (command, "signature-algorithm-name"))
     {
       const char *digest = ksba_cert_get_digest_algo (obj->cert);
       if (digest)
@@ -906,6 +937,35 @@ nasl_cert_query (lex_ctxt *lexic)
           retc->size = strlen (name);
         }
     }
+  else if (!strcmp (command, "public-key-algorithm-name"))
+    {
+      gnutls_datum_t datum;
+      gnutls_x509_crt_t cert = NULL;
+      int algo;
+      char *algo_name;
+
+      datum.data =
+        (void *) ksba_cert_get_image (obj->cert, (size_t *) &datum.size);
+      if (!datum.data)
+        return NULL;
+      if (gnutls_x509_crt_init (&cert) != GNUTLS_E_SUCCESS)
+        return NULL;
+      if (gnutls_x509_crt_import (cert, &datum, GNUTLS_X509_FMT_DER)
+          != GNUTLS_E_SUCCESS)
+        return NULL;
+      if ((algo = gnutls_x509_crt_get_pk_algorithm (cert, NULL)) < 0)
+        {
+          g_message ("%s: Error getting the public key algorithm name.",
+                     __func__);
+          return NULL;
+        }
+      algo_name = gnutls_pk_algorithm_get_name (algo)
+                    ? g_strdup (gnutls_pk_algorithm_get_name (algo))
+                    : g_strdup ("unknown");
+      retc = alloc_typed_cell (CONST_DATA);
+      retc->size = strlen (algo_name);
+      retc->x.str_val = algo_name;
+    }
   else if (!strcmp (command, "modulus"))
     {
       gnutls_datum_t datum, m, e;
@@ -925,7 +985,9 @@ nasl_cert_query (lex_ctxt *lexic)
 
       retc = alloc_typed_cell (CONST_DATA);
       retc->size = m.size;
-      retc->x.str_val = g_memdup (m.data, m.size);
+      retc->x.str_val = g_malloc0 (m.size);
+      memcpy (retc->x.str_val, m.data, m.size);
+
       gnutls_free (m.data);
       gnutls_free (e.data);
       gnutls_x509_crt_deinit (cert);
@@ -949,7 +1011,9 @@ nasl_cert_query (lex_ctxt *lexic)
 
       retc = alloc_typed_cell (CONST_DATA);
       retc->size = e.size;
-      retc->x.str_val = g_memdup (e.data, e.size);
+      retc->x.str_val = g_malloc0 (e.size);
+      memcpy (retc->x.str_val, e.data, e.size);
+
       gnutls_free (m.data);
       gnutls_free (e.data);
       gnutls_x509_crt_deinit (cert);
