@@ -26,6 +26,7 @@
 #include "nasl_crypto.h"
 #include "nasl_crypto2.h"
 #include "nasl_debug.h"
+#include "nasl_frame_forgery.h"
 #include "nasl_func.h"
 #include "nasl_global_ctxt.h"
 #include "nasl_host.h"
@@ -107,6 +108,9 @@ static init_func libfuncs[] = {
   {"recv_line", nasl_recv_line},
   {"send", nasl_send},
   {"socket_negotiate_ssl", nasl_socket_negotiate_ssl},
+  {"socket_check_ssl_safe_renegotiation",
+   nasl_socket_check_ssl_safe_renegotiation},
+  {"socket_ssl_do_handshake", nasl_socket_ssl_do_handshake},
   {"socket_get_cert", nasl_socket_get_cert},
   {"socket_get_ssl_version", nasl_socket_get_ssl_version},
   {"socket_get_ssl_ciphersuite", nasl_socket_get_ssl_ciphersuite},
@@ -132,6 +136,7 @@ static init_func libfuncs[] = {
   {"resolve_host_name", resolve_hostname},
   {"resolve_hostname_to_multiple_ips", resolve_hostname_to_multiple_ips},
   {"get_host_ip", get_host_ip},
+  {"get_local_mac_address_from_ip", nasl_get_local_mac_address_from_ip},
   {"same_host", nasl_same_host},
   {"TARGET_IS_IPV6", nasl_target_is_ipv6},
   {"get_host_open_port", get_host_open_port},
@@ -245,6 +250,10 @@ static init_func libfuncs[] = {
   {"forge_igmp_v6_packet", forge_igmp_v6_packet},
   {"send_packet", nasl_send_packet},
   {"send_v6packet", nasl_send_v6packet},
+  {"send_arp_request", nasl_send_arp_request},
+  {"forge_frame", nasl_forge_frame},
+  {"send_frame", nasl_send_frame},
+  {"dump_frame", nasl_dump_frame},
   {"pcap_next", nasl_pcap_next},
   {"send_capture", nasl_send_capture},
   {"MD2", nasl_md2},
@@ -415,6 +424,7 @@ static struct
   {"ENCAPS_TLSv1", OPENVAS_ENCAPS_TLSv1},
   {"ENCAPS_TLSv11", OPENVAS_ENCAPS_TLSv11},
   {"ENCAPS_TLSv12", OPENVAS_ENCAPS_TLSv12},
+  {"ENCAPS_TLSv13", OPENVAS_ENCAPS_TLSv13},
   {"ENCAPS_TLScustom", OPENVAS_ENCAPS_TLScustom},
   {"ENCAPS_MAX", OPENVAS_ENCAPS_MAX},
   {"TH_FIN", TH_FIN},
