@@ -2025,7 +2025,10 @@ get_icmp_v6_element (lex_ctxt *lexic)
           retc = alloc_typed_cell (CONST_DATA);
           retc->size = get_var_size_by_name (lexic, "icmp") - 40 - 8;
           if (retc->size > 0)
-            retc->x.str_val = g_memdup (&(p[40 + 8]), retc->size + 1);
+            {
+              retc->x.str_val = g_malloc0 (retc->size + 1);
+              memcpy (retc->x.str_val, &(p[40 + 8]), retc->size + 1);
+            }
           else
             {
               retc->x.str_val = NULL;
